/****************************************************
*                     Header File                   *
* ================================================= *
*                                                   *
*                                                   *
*                                                   *
*****************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/limits.h>
#include <sys/signal.h>


#define MAX_THREAD_NUMBER	3000
#define MAX_WRITER_NUMBER       100
#define MAX_READER_NUMBER       700
#define DEFAULT_NUM_READERS     5
#define DEFAULT_NUM_WRITERS     5
#define DEFAULT_SHMEM_SIZE      200000
#define MAX_THREAD		500
#define SHMEM_MODE              (SHM_R | SHM_W)


#define USAGE   "\nUsage: %s [-r num_readers] [-w num_writers] [-s shmem_size]\n \n" \
                "\t-r num_readers  number of reader threads to create\n" \
                "\t-w num_writers  number of writer threads to create\n" \
                "\t-s buffer_size  size of shmem segments in bytes\n"   \
                "\t                must be less than 256MB\n\n"



/*************** 
Global Variables 
****************/


pthread_t *writer_th;
pthread_t *reader_th;

int              thread_hold[MAX_WRITER_NUMBER];
pthread_mutex_t  cond_mutex[MAX_WRITER_NUMBER];
pthread_mutex_t  mutex_r[MAX_WRITER_NUMBER];
pthread_cond_t   cond_var[MAX_WRITER_NUMBER];

int      num_readers = DEFAULT_NUM_READERS;
int      num_writers = DEFAULT_NUM_WRITERS;

int           shmid;
char          *shmaddr;
int           buffer_size = DEFAULT_SHMEM_SIZE;
int           *read_count[MAX_WRITER_NUMBER];
char          *shmptr[MAX_THREAD_NUMBER];
unsigned long cksum[MAX_WRITER_NUMBER];
unsigned long *checksum[MAX_WRITER_NUMBER];


/*************** 
Global Functions 
****************/


void parse_args(int , char **);
void *reader(void *);
void *writer(void *);
void error(const char *, int);
void release();
