/************************************************************************
*                               Main ()                                 *
* ===================================================================== *
*  Date     : OCT 2009                                                  *
*  Developer: Subhankar Katyayan                                        *
*  Function : Main program returns (0)  on Successful Completion        * 
*                                  (-1) on Error Occured                * 
*                                                                       *
*                                                                       *
*                                                                       *
************************************************************************/

#include "header.h"


int main(int argc, char **argv)
{
	pthread_attr_t  newattr;

	int    i, j, k;
	size_t Size;
        unsigned long *ulptr;

/****************************************************
Parse Command Line Arguments and print program header
*****************************************************/
	
	parse_args(argc, argv);


        printf("%s: Shared Memory Test Program\n\n", *argv);

        printf("\tNumber of  writers thread   = %d\n", num_writers);
        printf("\tNumber of  readers thread   = %d\n", num_readers);
        printf("\tShared Memory Size in Bytes = %d\n", buffer_size);
        printf("\n");

	Size = buffer_size;  

/*************************************************************
Get The Shared Memory Segment and Attach it to current Process 
**************************************************************/

	if((shmid = shmget(IPC_PRIVATE, Size, SHMEM_MODE)) < 0)
		perror("Shmget Failed : ");


	if((shmaddr = (char *)shmat(shmid, 0, 0)) == (void *)-1){
		perror("Shmat Failed : ");
		exit(1);
	}		
	
	for(i = 0; i < num_writers; i++)
	{
		read_count[i] = (int *) shmaddr;

		*(read_count[i]) = 0;
	
		checksum[i] = (unsigned long *)shmaddr;

		ulptr = checksum[i]; 

		shmptr[i] = shmaddr;


		for(k = 0; k < num_readers; k++)
 		{
         		*ulptr = 0;
         		ulptr++;
 		}
	}
	
/*******************
Create threads array                                         
********************/

	writer_th = (pthread_t *) malloc((size_t) (num_writers \
                                            * sizeof(pthread_t)));
 	reader_th = (pthread_t *) malloc((size_t) (num_writers \
                              * num_readers * sizeof(pthread_t)));


 	for(i = 0; i < num_writers; i++) {

 	if(pthread_mutex_init(&mutex_r[i] , (pthread_mutexattr_t *)NULL) != 0)
        	 perror("Can't Initialise Mutex : ");


 	if(pthread_mutex_init (&cond_mutex[i], (pthread_mutexattr_t *)NULL)!=0)
        	 perror("Can't Initialize Cond_Mutex : ");


 	if(pthread_cond_init (&cond_var[i], (pthread_condattr_t *)NULL)!=0)
        	 perror("Cond_Var Failed : ");

/************************************
Lock the access to the shared segment 
*************************************/

 	thread_hold[i] = 1;
 	}

 	if(pthread_attr_init(&newattr))
  	      perror("Attr_Init Failed : ");


 	if(pthread_attr_setdetachstate(&newattr, PTHREAD_CREATE_JOINABLE)){
        	 perror("Attr_Setdetachstate Failed : ");
         	 printf("Successful\n");
 	}


/************************
Create All Writer Threads
*************************/

        for(i = 0; i < num_writers; i++)
        {

                if(pthread_create(&writer_th[i], &newattr, writer, \
                                                        (void *) (long)i))
                        perror("Writer Failed : ");
		

                k = i * num_readers;

/************************
Create All Reader Threads
*************************/

                for(j = k; j < (k + num_readers) ; j++)
                {
                        if(pthread_create(&reader_th[j], &newattr, reader,\
                                                        (void *) (long)j))

                        perror("Reader Failed : ");
                }
        }


        for(i = 0; i < num_writers; i++)
        {
               if(pthread_join(writer_th[i], NULL)) {
                        printf("writer_th: pthread_join return: %d\n",i);

                        perror("Pthread_join Bad Status : ");
                }

                k = i * num_readers;


                for(j = k; j < (k + num_readers) ; j++)
                {
                        if(pthread_join(reader_th[j], NULL)) {
                        perror("Pthread_join Bad Status : ");
                        }
                }
        }

        printf("\n\tMain : Thread Created Successfully\n");

        release();
        printf("\nsuccessful!\n");

        return(0);
}
